{
  Demonstration program for ChroMathsome.
  This program shows how to graph mathematical functions in Delphi.
  It also shows how one can implement functions like zooming (in and out) and how to
  scale the graph so that it fits the screen while keeping the correct X-Y coordinates.
  For function names and definitions, look in Calculus.txt.

  Additions/new features (vs. the program finished and loaded up on 18/09/'96):
  - added printing capabilities (21/09/'96)
  - new interface (22/09/'96)
  - saving to a (bitmap) file (22/09/'96)
  (Eureka...I've found it.
   The canvases of a form (component) and printer can be used the same way! :))

  Copyright 1996, John Merrit
  All Rights Reserved
  The reader may do with this code whatever he/she likes except...
  'Reproduction of this 'program' for commercial purposes is prohibited.'

  Most of this code was built as an experiment in programming functions
  to draw mathematical functions using Delphi.
  I haven't seen any other code around that does this and maybe
  someone can get any benefits out of it -> you may have it.
  ( I wouldnt mind a little credit... After all, I wrote this program )

  I would appreciate:
  *  Comments, Suggestions, Bug finds (read the comments throughout this code
     before you call something a 'bug').
  *  If you improve this code, please send me a copy of the changes.
  What I do not want is messages starting with "I am new to Delphi...".
  I simply do not have time to support this demoprogram.
  Write to me if you have found a problem, not if you have a problem
  which can be answered by posting to the comp.lang.delphi.components
  newsgroup or in one of the mathematics/computer science/numerical analysis newsgroups.

  Correspondence: John.Merrit@Net4All.BE

  Disclaimer:
  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.                                        }


unit mdraw2;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, ExtCtrls, Printers;

type

  avector = array[1..1500] of extended;  { Storage for values of function to draw.}
  { Use 1500 so that the user can resize up to resolution 1500.  }

  TForm1 = class(TForm)
    DrawPad: TPaintBox;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Save1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    FunctionEdit: TEdit;
    Label1: TLabel;
    Bevel1: TBevel;
    Graph1: TMenuItem;
    Draw2: TMenuItem;
    N2: TMenuItem;
    ZoomIn1: TMenuItem;
    ZoomOut1: TMenuItem;
    Print1: TMenuItem;
    N3: TMenuItem;
    Options1: TMenuItem;
    CurveColor1: TMenuItem;
    AxesColor1: TMenuItem;
    ColorDialog1: TColorDialog;
    BackgroundColor1: TMenuItem;
    Image1: TImage;
    SaveDialog1: TSaveDialog;
    RedrawLastCurve1: TMenuItem;
    N4: TMenuItem;
    ShowCordinates1: TMenuItem;
    procedure Exit1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure Draw2Click(Sender: TObject);
    procedure DrawPadPaint(Sender: TObject);
    procedure ZoomIn1Click(Sender: TObject);
    procedure ZoomOut1Click(Sender: TObject);
    procedure ChangeColor(Sender: TObject);
    procedure Print1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure RedrawLastCurve1Click(Sender: TObject);
    procedure ShowCordinates1Click(Sender: TObject);
    procedure DrawPadMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private declarations }
    procedure DrawAxes;
    procedure PlotGraph(Surface: integer);

  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  ScaleY, ZoomFactor: integer;
  CurveColor, AxesColor, BackgroundColor: TColor;

implementation

uses Calculus;

{$R *.DFM}

procedure TForm1.Exit1Click(Sender: TObject);
begin
     Close;
end;

procedure TForm1.DrawAxes;
var i, x, y: integer;
begin
     x:= DrawPad.width;
     y:= DrawPad.height;
     with DrawPad.Canvas do
     begin
     Pen.Color:= AxesColor;
     for i:= 1 to x - 1 do
     begin
     moveto(i,Round(y/2));
     lineto(i+1,Round(y/2));
     end;
     for i:= 1 to y - 1 do
     begin
     moveto(Round(x/2),i);
     lineto(Round(x/2),i+1);
     end;
     end;
end;


procedure TForm1.FormActivate(Sender: TObject);
begin
     ScaleY:= Round(DrawPad.Height/10);
     ZoomFactor:= 0;
     CurveColor:= clBlue;
     AxesColor:= clRed;
     BackgroundColor:= clWhite;
end;

{ Draw a userdefined function on the drawpad/print it out or save it in a
  (bitmap) file.
  First step is to calculate the function values (stored in v)
  and next use v to draw the function.
  This approach makes the drawing of complex functions (e.g. exp(cos(sinh(2 + x))) - tan(x))
  more fluently.
  Note that if you resize the drawpad, the graph will automatically adjust itself to the new screensize
  (this is also true for the printed and saved curve).
  Take a look at some strange beautifull graphs:
  - sin(x*x*x) ; the function when you start this application
  - cos(x*x*x)
  - 2*exp(-sin(5*x))*cos(3*x) ( a damped oscillating system with sinusodal damping factor?)
  - etc...
  Note also that if your function leads to an invalid operation, a message box will inform you so.
  For example: trying to draw the function sqrt(x) without modifying the code will result in
  an invalid operation (why? -> study the code).
  The surface variable (for ease we take an integer value) determines where the
  curve needs to be drawed (1 = screen; 2 = printer; 3 = file).
  To print a curve, we re-direct the output to the printer's canvas
  instead of the drawpad.
  Printed using a color printer will draw with the same colors as on the
  screen.
  Black and white (laser) printers will only use colors ...
  Note that a Paintbox component (used as screen-interface) does not have
  printing or saving facilities.
  Printing in Delphi (except for the contents of a memo) is not just
  putting a control on a form!
  The code below draws both the curve and the axes on the printer's
  canvas after calculation of scalefactors px and py.
  Px and py are needed because the printer's canvas is not the same
  as the canvas on a form (resolution!).
  Set px (and py) equal to one and look at the resulting printed page if
  you don't believe this.
  To save a curve to a (bitmap) file, we re-direct the output to a TImage's canvas
  instead of the drawpad.
  Only the curve (and the axes) will be printed/saved, not the whole form.
  Note that this code is only a demonstration (it works!).
  The wistles and bells are your part.                                 }
procedure TForm1.PlotGraph(Surface: integer);
var i,x,y,px,py: integer;
    fx: extended;
    v: ^avector;
    UserFunc: string;
begin
     new(v);
     try
     UserFunc:= FunctionEdit.Text;
     Form1.Caption:= 'f(x) = ' + FunctionEdit.Text + ' Zoom factor: ' + IntToStr(ZoomFactor);
     { procedure PrepFunction adds closing brackets (')') if nescessary. }
     PrepFunction(UserFunc);
     x:= DrawPad.width;
     y:= DrawPad.height;
     try
     for i:= 1 to x - 1 do
     begin
     { x:= 0 at 1/2 width drawpad. fx = x- value where function needs to be evaluated. }
     fx:= (i - x/2)/ScaleY;
     v^[i]:= FunctionValue(UserFunc,fx);
     end;

     case Surface of
     1:begin { Screen = DrawPad }
     DrawAxes;
     with DrawPad.Canvas do
     begin
     Color:= BackgroundColor;
     Pen.Color := CurveColor;
     for i:= 1 to x - 2 do
     begin
     moveto(i,Round(y/2 - ScaleY*v^[i]));
     lineto(i+1,Round(y/2 - ScaleY*v^[i+1]));
     end;
     end;
     end;
     2:begin { Printer }
     with Printer do
     begin

     Title := 'f(x) = ' + FunctionEdit.Text;
     Orientation := poLandscape;

     BeginDoc;	{ start printing }

     with Printer.Canvas do
     begin
     { Adjust scale factors so the printed curve spreads over the
       printed page.                                             }
     px:= round(PageWidth/x);
     py:= round(PageHeight/y);

     { Draw Axes on the print page. }
     Pen.Color := AxesColor;
     for i:= 1 to x - 1 do
     begin
     moveto(px*i,Round(py*y/2));
     lineto(px*i+1,Round(py*y/2));
     end;
     for i:= 1 to y - 1 do
     begin
     moveto(Round(px*x/2),py*i);
     lineto(Round(px*x/2),py*i+1);
     end;

     { Print the curve. }
     Pen.Color := CurveColor;
     for i:= 1 to x - 2 do
     begin
     moveto(px*i,Round(py*y/2 - py*ScaleY*v^[i]));
     lineto(px*i+1,Round(py*y/2 - py*ScaleY*v^[i+1]));
     end;

     Font.Height:= py*14;
     TextOut(PageWidth div 2,
     PageHeight - 150,'f(x) = ' + FunctionEdit.Text);

     end;
     EndDoc;	{ finish printing }
     end;

     end;
     3:begin { Save to a (bitmap) file }
     with Image1.Canvas do
     begin
     Color:= BackgroundColor;
     { Draw Axes. }
     Pen.Color := AxesColor;
     for i:= 1 to x - 1 do
     begin
     moveto(i,Round(y/2));
     lineto(i+1,Round(y/2));
     end;
     for i:= 1 to y - 1 do
     begin
     moveto(Round(x/2),i);
     lineto(Round(x/2),i+1);
     end;

     { Draw the curve. }
     Pen.Color := CurveColor;
     for i:= 1 to x - 2 do
     begin
     moveto(i,Round(y/2 - ScaleY*v^[i]));
     lineto(i+1,Round(y/2 - ScaleY*v^[i+1]));
     end;
     { The saving function ->  procedure Save1Click(Sender: TObject); }
     end;
     end;
     end;
     except on EInvalidOp do showmessage('An error has occured!');
     end;
     finally  { ...make sure the pointer gets freed!!!  }
     dispose(v);
     end;
end;


procedure TForm1.Draw2Click(Sender: TObject);
begin
    PlotGraph(1);
end;

procedure TForm1.DrawPadPaint(Sender: TObject);
begin
      PlotGraph(1);
end;

procedure TForm1.ZoomIn1Click(Sender: TObject);
begin
     ScaleY:= 2*ScaleY;
     ZoomFactor:= ZoomFactor + 1;
     DrawPad.Refresh;
     Draw2Click(Sender);
end;

procedure TForm1.ZoomOut1Click(Sender: TObject);
begin
     ScaleY:= round(ScaleY/2);
     ZoomFactor:= ZoomFactor - 1;
     DrawPad.Refresh;
     Draw2Click(Sender);
end;

procedure TForm1.ChangeColor(Sender: TObject);
begin
     if ColorDialog1.Execute then
     begin
     if Sender = CurveColor1 then
     CurveColor:= ColorDialog1.Color
     else if Sender = AxesColor1 then
     AxesColor:= ColorDialog1.Color
     else BackgroundColor:= ColorDialog1.Color;
     end;
     DrawPad.Refresh;
     Draw2Click(Sender);
end;

procedure TForm1.Print1Click(Sender: TObject);
begin
     PlotGraph(2);
end;

procedure TForm1.Save1Click(Sender: TObject);
begin
     if SaveDialog1.Execute then
     begin
     PlotGraph(3);
     Image1.Picture.SaveToFile(SaveDialog1.FileName);
     Image1.Visible:= false; { For esthetical reasons... }
     end;

end;

procedure TForm1.RedrawLastCurve1Click(Sender: TObject);
begin
     DrawPad.Refresh;
end;

procedure TForm1.ShowCordinates1Click(Sender: TObject);
begin
     if ShowCordinates1.Checked then
     ShowCordinates1.Checked:= false
     else
     ShowCordinates1.Checked:= true;
end;

procedure TForm1.DrawPadMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var dx,dy: real;
begin
   dx:= X - DrawPad.Width/2;
   dy:= DrawPad.Height/2 - Y;
   if ShowCordinates1.Checked then
   DrawPad.Canvas.TextOut(5, DrawPad.Height - 20, '(' + FloatToStr(dx/ScaleY) + ';' +
   FloatToStr(dy/ScaleY) + ')     ');
end;

end.
